'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_1 = require('tslib');
var immutable = require('immutable');
var isPlainObject = _interopDefault(require('is-plain-obj'));
var slate = require('slate');
var slateKitPluginsUtils = require('@vericus/slate-kit-plugins-utils');
var tinycolor = _interopDefault(require('tinycolor2'));
var Renderer = _interopDefault(require('@vericus/slate-kit-highlight-text-renderer'));

var defaultOptions = {
    type: undefined,
    styles: undefined,
    data: undefined,
    defaultColor: undefined,
    alpha: 1,
    externalRenderer: false
};
var Options = /** @class */ (function (_super) {
    tslib_1.__extends(Options, _super);
    function Options() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Options.create = function (attrs) {
        if (attrs === void 0) { attrs = {}; }
        if (Options.isOptions(attrs))
            return attrs;
        if (isPlainObject(attrs))
            return Options.fromJSON(attrs);
        throw new Error("`Options.create` only accepts objects, strings or options");
    };
    Options.fromJSON = function (object) {
        if (Options.isOptions(object))
            return object;
        var type = object.type, data = object.data, styles = object.styles, defaultColor = object.defaultColor, _a = object.alpha, alpha = _a === void 0 ? 1 : _a;
        if (!type || !data || !styles || !defaultColor || !Array.isArray(styles)) {
            throw new Error("Highlight text require type, data and style option");
        }
        if (alpha < 0 || alpha > 1) {
            throw new Error("Alpha value can only go from 0 to 1");
        }
        return new Options({
            type: type,
            data: data,
            styles: styles,
            alpha: alpha,
            defaultColor: defaultColor
        });
    };
    Options.isOptions = function (args) {
        return !!(args instanceof immutable.Record &&
            ["type", "data", "styles", "defaultColor"].every(function (key) { return args.has(key); }));
    };
    return Options;
}(immutable.Record(defaultOptions, "highlight text option")));

function createChanges(opt) {
    var defaultColor = opt.defaultColor, data = opt.data, type = opt.type;
    return {
        changeColor: function (change, color) {
            var _a, _b;
            var value = change.value;
            var selection = value.selection, document = value.document;
            if (slateKitPluginsUtils.hasMark(value, type)) {
                if (tinycolor(color).toName() === defaultColor) {
                    if (selection.isCollapsed) {
                        slateKitPluginsUtils.removeCollapsedMark(value, change, type);
                    }
                    else {
                        slateKitPluginsUtils.removeExpandedMark(document, selection, change, type);
                    }
                }
                else {
                    var newMark = new slate.Mark({
                        type: type,
                        data: slate.Data.create((_a = {},
                            _a[data] = color,
                            _a))
                    });
                    if (selection.isCollapsed) {
                        slateKitPluginsUtils.removeCollapsedMark(value, change, type);
                        change.focus();
                        change.addMark(newMark);
                    }
                    else {
                        slateKitPluginsUtils.removeExpandedMark(document, selection, change, type);
                        change.addMarkAtRange(selection, newMark);
                    }
                }
            }
            else if (tinycolor(color).toName() !== defaultColor) {
                var mark = new slate.Mark({
                    type: type,
                    data: slate.Data.create((_b = {},
                        _b[data] = color,
                        _b))
                });
                if (selection.isCollapsed) {
                    change.addMark(mark);
                }
                else {
                    slateKitPluginsUtils.removeExpandedMark(document, selection, change, type);
                    change.addMarkAtRange(selection, mark);
                }
            }
            change.focus();
            return change;
        }
    };
}

function createUtils(opts) {
    var defaultColor = opts.defaultColor, data = opts.data, type = opts.type;
    return {
        currentColor: function (value) {
            if (value.selection && value.selection.isFocused && value.activeMarks) {
                var activeColorMarks = value.activeMarks
                    .filter(function (mark) { return mark.type === type; })
                    .map(function (mark) { return mark.data.get(data); });
                var uniqueColorMarks = Array.from(new Set(activeColorMarks)).slice();
                if (uniqueColorMarks.length !== 0) {
                    return uniqueColorMarks[0];
                }
            }
            return defaultColor;
        }
    };
}

function createPlugin(pluginOptions) {
    var options = Options.create(pluginOptions);
    var changes = createChanges(options);
    var utils = createUtils(options);
    var plugin = {
        options: options,
        changes: changes,
        utils: utils
    };
    if (!options.externalRenderer) {
        var renderMark = Renderer(options).renderMark;
        plugin.renderMark = renderMark;
    }
    return plugin;
}

exports.default = createPlugin;
//# sourceMappingURL=slate-kit-highlight-text.js.map
